>From a48de97f593b17160487e67f953c8d2b25f9e98a Mon Sep 17 00:00:00 2001
From: Craig Ringer <craig@2ndquadrant.com>
Date: Fri, 16 May 2014 13:29:16 +0800
Subject: [PATCH] Don't restart BGW_NO_RESTART workers after postmaster crash

ResetBackgroundWorkerCrashTimes was ignoring BGW_NO_RESTART and clearing the
crash time of all bgworkers on postmaster restart so they'd be restarted as
soon as the postmaster was up and ready. It should only do this for workers
that are supposed to be restarted, as a BGW_NO_RESTART worker with no
rw_crashed_at set is assumed to be newly registered.

As a result, if a bgworker registered another BGW_NO_RESTART bgworker
during postmaster restart we'd get an increasing number of duplicates
of the BGW_NO_RESTART worker every time the postmaster restarted.
---
 src/backend/postmaster/bgworker.c | 3 ++-
 1 file changed, 2 insertions(+), 1 deletion(-)

diff --git a/src/backend/postmaster/bgworker.c b/src/backend/postmaster/bgworker.c
index 85a3b3a..4bbebb6 100644
--- a/src/backend/postmaster/bgworker.c
+++ b/src/backend/postmaster/bgworker.c
@@ -411,7 +411,8 @@ ResetBackgroundWorkerCrashTimes(void)
 		RegisteredBgWorker *rw;
 
 		rw = slist_container(RegisteredBgWorker, rw_lnode, iter.cur);
-		rw->rw_crashed_at = 0;
+		if (rw->rw_worker.bgw_restart_time != BGW_NEVER_RESTART)
+			rw->rw_crashed_at = 0;
 	}
 }
 
-- 
1.9.0

